# Marktforschung-Buch, Abb. 100 Merkmalsraum sowie 104 und 105 ROC-Kurven
library(MASS); library(ROCR); library(hmeasure)
library(tree); library(maptree); library(partykit)
library(nnet); library(ggrepel)  
setwd("C:/Users/Alpha/Desktop/R")
Data=read.csv("Beispiel1.csv",header=TRUE,sep=";",dec=",")

# Aufteilen der Daten in Train (fr fit) und Test (fr predict)
set.seed(1234)
n=dim(Data)[1]; n2=round(0.8*n); tr=sample(1:n,n2)
Data.train=Data[tr,]; Data.test=Data[-tr,]

pdf(file="pdf/Abb100.pdf",width=11.7/2.54, height=11.7/2.54)
ggplot(Data.train,aes(Alter,Einkommen,label=Besteller)) +
  geom_point(shape=15+Data.train$Besteller, size=1.5) +
  geom_vline(xintercept=29,linetype="longdash",size=0.25)+
  geom_vline(xintercept=40.5,linetype="longdash",size=0.25)+
  geom_text_repel(parse=TRUE,size=3,box.padding=0.15,
                  point.padding=0.1) +
  theme_classic(base_size=9) +
  xlab("Alter (in Jahren)") + ylab("Einkommen (in 1.000  pro Jahr)") +
  theme(axis.title.x = element_text(size=9)) +
  theme(axis.text.x = element_text(size=9)) +
  theme(axis.line.x = element_line(size=0.25)) + 
  theme(axis.line.y = element_line(size=0.25)) + 
  theme(axis.ticks = element_line(size=0.25)) + 
  theme(axis.title.y = element_text(size=9)) + 
  theme(axis.text.y = element_text(size=9)) +
  expand_limits(x=0, y=0) + expand_limits(x=70, y=110) + 
  scale_x_continuous(expand = c(0, 0)) + 
  scale_y_continuous(expand = c(0, 0)) 
dev.off()

# Gini
sum(Data.train$Besteller)
P1=20/29; P1
P2=9/29; P2
I0=1-P1**2-P2**2; I0
I1=0
P1=10/19; P1
P2=9/19; P2
I2=1-P1**2-P2**2; I2

dI=I0-10/29*I1-19/29*I2; dI

Data.train=Data.train[,c(2,3,8)]
Data.train$Besteller=as.factor(Data.train$Besteller)
Data.test=Data.test[,c(2,3,8)]
Data.test$Besteller=as.factor(Data.test$Besteller)
Data.formula=Besteller~Alter+Einkommen

# Diskriminanzanalyse
lda.fit=lda(Data.formula,Data.train); lda.fit
lda.scores.train=predict(lda.fit,Data.train)$posterior[, 2]
lda.scores.test=predict(lda.fit,Data.test)$posterior[, 2]

# Entscheidungsbaum mit tree-package
tree.fit=tree(Data.formula,data=Data.train,method="recursive.partition", 
              split="gini",mincut=8,mindev=0)
draw.tree(tree.fit, pch=par("pch"), 
          nodeinfo=TRUE, units="Anteil", cases="Personen", digits=4,
          print.levels=TRUE, new = TRUE)
tree.scores=predict(tree.fit,Data)[,2]
tree.scores.train=predict(tree.fit,Data.train)[,2]
tree.scores.test=predict(tree.fit,Data.test)[,2]

# Entscheidungsbaum mit party-package
ctree.fit=ctree(Data.formula, data=Data.train,maxdepth=2,
                mincriterion=0,minsplit=0,minbucket=0)
plot(ctree.fit,gp=gpar(fontsize=10)) 
a=predict(ctree.fit,Data.train,type="prob"); n=dim(Data.train)[1]
ctree.scores.train=1:n; for (i in 1:n){ctree.scores.train[i]=a[i,2]}
a=predict(ctree.fit,Data.test,type="prob"); n=dim(Data.test)[1]
ctree.scores.test=1:n; for (i in 1:n){ctree.scores.test[i]=a[i,2]}

# Neuronales Netz mit nnet-package
nnet.fit <- nnet(Data.formula,Data.train,size=15,maxit=300)
nnet.fit
nnet.scores.train=predict(nnet.fit,Data.train)
nnet.scores.test= predict(nnet.fit,Data.test)

# Plots und AUC-Werte fr train
scores=data.frame(LDA=lda.scores.train,Tree=tree.scores.train,
        CTree=ctree.scores.train,NNet=nnet.scores.train)
results=HMeasure(Data.train$Besteller,scores)
plotROC(results)
results$metrics
# Graphische Ausgabe
Methode=c("lda","tree","ctree","nnet","Zufall")
Data=as.data.frame(matrix(NA,nrow=2,ncol=3))
names(Data)=c("FPR","TPR","Methode")
Data[1,1]=0; Data[1,2]=0; Data[2,1]=1; Data[2,2]=1;
Data$Methode=Methode[5] 
for (i in 1:4){
  a=attr(x=results,which="data")    
  F0=a[[i]]$F0; F1=a[[i]]$F1; FPR=1-F0; TPR=1-F1
  n=length(FPR)
  Data1=as.data.frame(matrix(Methode[i],nrow=n,ncol=3))
  names(Data1)=names(Data); Data1[,1]=FPR; Data1[,2]=TPR
  Data=rbind(Data,Data1)
}
Data=Data[order(Data$FPR,Data$TPR),]
Data$Methode=factor(Data$Methode,levels=c("lda","tree","ctree","nnet","Zufall"))
pdf(file="pdf/Abb104.pdf",width=11.7/2.54, height=11.7/2.54)
ggplot(data=Data,aes(x=FPR,y=TPR,group=Methode)) +
  geom_line(aes(linetype=Methode),size=0.3)+
  theme_classic(base_size=9) +
  theme_minimal()+
  theme(axis.title.x = element_text(size=9)) +
  theme(axis.text.x = element_text(size = 9)) +
  theme(axis.title.y = element_text(size=9)) + 
  theme(axis.text.y = element_text(size = 9)) +
  labs(title="",x="Falsch-Positiv-Rate (FP/N)",y="Wahr-Positiv-Rate (TP/P)",legend="Legend") +
  scale_linetype_manual(values=c(4,5,2,1,3)) +
  theme(legend.title = element_text(size = 9)) + 
  theme(legend.text = element_text(size = 9)) +
  theme(legend.position="bottom")
dev.off()

# Plots und AUC-Werte fr test
scores=data.frame(LDA=lda.scores.test,Tree=tree.scores.test,
                  CTree=ctree.scores.test,NNet=nnet.scores.test)
results=HMeasure(Data.test$Besteller,scores)
plotROC(results)
results$metrics
# Graphische Ausgabe
Data=as.data.frame(matrix(NA,nrow=2,ncol=3))
names(Data)=c("FPR","TPR","Methode")
Data[1,1]=0; Data[1,2]=0; Data[2,1]=1; Data[2,2]=1;
Data$Methode=Methode[5] 
for (i in 1:4){
  a=attr(x=results,which="data")    
  F0=a[[i]]$F0; F1=a[[i]]$F1; FPR=1-F0; TPR=1-F1
  n=length(FPR)
  Data1=as.data.frame(matrix(Methode[i],nrow=n,ncol=3))
  names(Data1)=names(Data); Data1[,1]=FPR; Data1[,2]=TPR
  Data=rbind(Data,Data1)
}
Data=Data[order(Data$FPR,Data$TPR),]
Data$Methode=factor(Data$Methode,levels=c("lda","tree","ctree","nnet","Zufall"))
pdf(file="pdf/Abb105.pdf",width=11.7/2.54, height=11.7/2.54)
ggplot(data=Data,aes(x=FPR,y=TPR,group=Methode)) +
  geom_line(aes(linetype=Methode),size=0.3)+
  theme_classic(base_size=9) +
  theme_minimal()+
  theme(axis.title.x = element_text(size=9)) +
  theme(axis.text.x = element_text(size = 9)) +
  theme(axis.title.y = element_text(size=9)) + 
  theme(axis.text.y = element_text(size = 9)) +
  labs(title="",x="Falsch-Positiv-Rate (FP/N)",y="Wahr-Positiv-Rate (TP/P)",legend="Legend") +
  scale_linetype_manual(values=c(4,5,2,1,3)) +
  theme(legend.title = element_text(size = 9)) + 
  theme(legend.text = element_text(size = 9)) +
  theme(legend.position="bottom")
dev.off()
